# -*- coding: utf-8 -*-
# Author: ZKH
# Date：2021/3/20
from yqn_project_cli.utils.core.as_flask import JSONResponse
from config import ProjectConfig


def heartbeats(*args, **kwargs):
    return JSONResponse(
        data={'app_id': ProjectConfig.id, 'app_status': "OK", 'engine_version': ProjectConfig.version_int}
    )



# -*- coding: utf-8 -*-
# Author: ZKH
# Date：2021/2/24
import os
from yqn_project_cli.utils import conf_loader


class ProjectConfig:
    id = "${app_id}"
    name = "${project_name}"

    version = '1.0.0'
    version_int = 1 * 10000 + 0 * 100 + 0 * 1

    heartbeat_url = '/heartbeat/'
    heartbeat_urls = [heartbeat_url, '/actuator/prometheus/', '/actuator/info/', '/info/']

    project_base_dir = os.path.dirname(os.path.dirname(__file__))
    project_father_dir = os.path.dirname(project_base_dir)

    resources = os.path.join(project_base_dir, 'resources')  # resource_path
    outputs = os.path.join(project_base_dir, 'outputs')  # output_path

    hot_update = False  # whether service hot-update or not


class LoggerConfig:
    app_id = ProjectConfig.id
    app_name = ProjectConfig.name
    log_path = '/app/' + app_name + '/log/'
    try:
        os.makedirs(log_path, exist_ok=False)
    except Exception as e:
        pass


local_conf_loader = conf_loader(os.path.join(ProjectConfig.project_father_dir, '.{}_config.py'.format(ProjectConfig.name)))


class ApolloClientConfig:
    apollo_meta_server_url = local_conf_loader('APOLLO_META_SERVER_URL', 'http://192.168.10.227:8080')
    apollo_cluster = local_conf_loader('APOLLO_CLUSTER', "qa")
    apollo_app_id = local_conf_loader('APOLLO_APP_ID', ProjectConfig.id)
    apollo_app_name = local_conf_loader('APOLLO_APP_NAME', ProjectConfig.name)
    apollo_env = local_conf_loader('APOLLO_ENV', "FAT")
    apollo_namespace = local_conf_loader('APOLLO_NAMESPACE', "application")


class ODPSConfig:
    odps_access_id = local_conf_loader('ODPS_ACCESS_ID', '')
    odps_secret_access_key = local_conf_loader('ODPS_SECRET_ACCESS_KEY', '')
    odps_project = local_conf_loader('ODPS_PROJECT', '')
    odps_endpoint = local_conf_loader('ODPS_ENDPOINT', '')


class MySQLConfig:
    mysql_engine = local_conf_loader('MYSQL_ENGINE', '')
    mysql_username = local_conf_loader('MYSQL_USERNAME', '')
    mysql_password = local_conf_loader('MYSQL_PASSWORD', '')
    mysql_host = local_conf_loader('MYSQL_HOST', '')
    mysql_port = local_conf_loader('MYSQL_PORT', '')
    mysql_charset = local_conf_loader('MYSQL_CHARSET', '')
    mysql_database = local_conf_loader('MYSQL_DATABASE', '')


class ESConfig:
    es_host = local_conf_loader('ES_HOST', '')
    es_port = local_conf_loader('ES_PORT', '')
    es_username = local_conf_loader('ES_USERNAME', '')
    es_password = local_conf_loader('ES_PASSWORD', '')


class RedisConfig:
    redis_host = local_conf_loader('REDIS_HOST', '')
    redis_port = local_conf_loader('REDIS_PORT', '')
    redis_db = local_conf_loader('REDIS_DB', '')
    redis_password = local_conf_loader('REDIS_PASSWORD', '')



# -*- coding: utf-8 -*-
# Author: ZKH
# Date：2021/2/26
from flask_restx import Resource
from yqn_project_cli.utils.restx import DANamespace
from yqn_project_cli.utils.core.as_flask import (
    UrlBase,
    BlueprintBase,
    View,
)


class ApiUrl(UrlBase):
    pass


class ApiBlueprint(BlueprintBase):
    pass


class ApiResource:
    def __init__(self, url, resource, title, **kwargs):
        assert isinstance(resource, (Resource, View)) \
               or issubclass(getattr(resource, 'view_class', None), (Resource, View)), \
            '%s is not subclass or instance of Resource or View' % type(resource)

        self.url = url
        self.resource = resource
        self.title = title
        self.kwargs = kwargs


class ApiNamespace:
    def __init__(self, namespace, resources, title, **kwargs):
        assert isinstance(namespace, DANamespace), '%s is not instance of DANamespace' % type(namespace)
        self.namespace = namespace
        self.resources = resources
        self.title = title
        self.kwargs = kwargs
        self.add_resources(resources)

    def add_resources(self, resources):
        for res in resources:
            assert isinstance(res, ApiResource), '%s is not instance of ApiResource' % type(res)
            view_class = getattr(res.resource, 'view_class', None)
            if view_class is not None and issubclass(view_class, (Resource, View)):
                res.resource = view_class

            self.namespace.add_resource(res.resource, res.url, **res.kwargs)



# -*- coding: utf-8 -*-
# Author: ZKH
# Date：2021/3/1
from flask import current_app
from yqn_project_cli.utils import error_mark
from yqn_project_cli.utils.restx import add_view_mth_resource
from yqn_project_cli.utils.core.as_flask import RoutePluginBase
from yqn_project_cli.utils.core.exceptions import (
    APIException,
    PortalException,
    HTTPException
)
from yqn_project_cli.rpc import RPCConfigBase


def general_error_handler(e):
    if isinstance(e, (APIException, PortalException)) or current_app.config['DEBUG']:
        return e

    elif isinstance(e, HTTPException):
        return APIException(errmsg=e.description, code=e.code)

    else:
        error_mark(e)  # log it
        return PortalException()


# add resource type
def flask_restx_tail(plugin):
    import os
    import json
    import importlib
    from flask_restx import Namespace
    from yqn_project_cli.utils import load_module_from_pyfile
    from yqn_project_cli.utils.restx import DANamespace

    routes = json.load(
        open(os.path.join(plugin.app.project_config.project_base_dir,
                          'config/{}.json'.format(plugin.app.project_config.name)), 'r'))

    for route in routes['path_list']:
        # views = load_module_from_pyfile(os.path.join(api_path, module, 'views.py'))
        views = importlib.import_module('api.{}.views'.format(route['module']))
        # importlib.reload(views)

        for k, v in views.__dict__.items():
            if isinstance(v, DANamespace):

                add_view_mth_resource(v, getattr(getattr(views, route['view_cls']), route['view_mth']), route)

                plugin.add_namespace(v)

                break  # only allow one namespace

            elif isinstance(v, Namespace):
                raise ValueError('%s need inheriting from %s, rather than %s ' % (v, DANamespace, Namespace))


# decorator type
def flask_restx_tail_old(plugin):
    import os
    import importlib
    from flask_restx import Namespace
    from yqn_project_cli.utils import load_module_from_pyfile
    from yqn_project_cli.utils.restx import DANamespace

    api_path = os.path.join(plugin.app.project_config.project_base_dir, 'api')
    for module in os.listdir(api_path):
        if module.startswith(('_', '.')) \
                or not os.path.isdir(os.path.join(api_path, module)) \
                or not os.path.exists(os.path.join(api_path, module, 'views.py')):
            continue

        # views = load_module_from_pyfile(os.path.join(api_path, module, 'views.py'))
        views = importlib.import_module('api.{}.views'.format(module))
        # importlib.reload(views)

        for k, v in views.__dict__.items():
            if isinstance(v, DANamespace):
                plugin.add_namespace(v)

            elif isinstance(v, Namespace):
                raise ValueError('%s need inheriting from %s, rather than %s ' % (v, DANamespace, Namespace))


def close_rpc_services(e):
    for name, server in current_app.services.items():
        if name != (current_app.server_manager and current_app.server_manager[0]) and isinstance(server, RPCConfigBase):
            server.close()



# -*- coding: utf-8 -*-
# Author: ZKH
# Date：2021/3/1
from threading import Lock
import pymysql
from pymysql.cursors import Cursor
from dbutils.pooled_db import PooledDB
from yqn_project_cli.rpc import RPCConfigBase
from yqn_project_cli.utils import (
    error_mark,
    warning_mark,
)


class DADictCursor(Cursor):
    dict_type = dict

    def _do_get_result(self):
        super(DADictCursor, self)._do_get_result()
        fields = []
        if self.description:
            for f in self._result.fields:
                fields.append(f.table_name + "." + f.name if f.table_name != f.org_table else f.name)

            self._fields = fields

        if fields and self._rows:
            self._rows = [self._conv_row(r) for r in self._rows]

    def _conv_row(self, row):
        if row is None:
            return None
        return self.dict_type(zip(self._fields, row))


class MySQLClient(RPCConfigBase):
    def __init__(self, **kwargs):
        super(MySQLClient, self).__init__(**kwargs)
        self.mysql_username = None
        self.mysql_password = None
        self.mysql_host = None
        self.mysql_port = None
        self.mysql_database = None
        self.mysql_charset = None

    def connect(self, **kwargs):
        if self.connector is None:
            self.mysql_username = self.server_manager.get_value('mysql_username') if self.use_conf_management \
                else self.client_config['mysql_username']

            self.mysql_password = self.server_manager.get_value('mysql_password') if self.use_conf_management \
                else self.client_config['mysql_password']

            self.mysql_host = self.server_manager.get_value('mysql_host') if self.use_conf_management \
                else self.client_config['mysql_host']

            self.mysql_port = self.server_manager.get_value('mysql_port') if self.use_conf_management \
                else self.client_config['mysql_port']

            self.mysql_database = self.server_manager.get_value('mysql_database') if self.use_conf_management \
                else self.client_config['mysql_database']

            self.mysql_charset = self.server_manager.get_value('mysql_charset') if self.use_conf_management \
                else self.client_config['mysql_charset']

            self.connector = PooledDB(
                creator=pymysql,
                maxconnections=6,
                mincached=2,
                maxcached=5,
                maxshared=3,
                blocking=True,
                maxusage=None,
                setsession=[],
                ping=0,
                host=self.mysql_host,
                port=int(self.mysql_port),
                user=self.mysql_username,
                password=self.mysql_password,
                database=self.mysql_database,
                charset=self.mysql_charset
            )

        connector = self.connector.connection()
        cursor = connector.cursor(cursor=kwargs.get('cursor_cls', Cursor))

        return connector, cursor

    def reconnect(self, **kwargs):
        mysql_username = kwargs.get('mysql_username', False)
        mysql_password = kwargs.get('mysql_password', False)
        mysql_host = kwargs.get('mysql_host', False)
        mysql_port = kwargs.get('mysql_port', False)
        mysql_database = kwargs.get('mysql_database', False)
        mysql_charset = kwargs.get('mysql_charset', False)

        if not all([mysql_username, mysql_password, mysql_host, mysql_port, mysql_database, mysql_charset]):
            return False

        if all([
            self.mysql_username == mysql_username,
            self.mysql_password == mysql_password,
            self.mysql_host == mysql_host,
            self.mysql_port == mysql_port,
            self.mysql_database == mysql_database,
            self.mysql_charset == mysql_charset,
        ]):
            return False

        else:
            with Lock():
                if self.connector is not None:
                    self.connector.close()
                    self.connector = None

                warning_mark('mysql is reconnecting ... ')
                print('old:', self)
                self.connect()
                print('new:', self)

            return True

    @staticmethod
    def close(connector=None, cursor=None):
        if cursor is not None:
            cursor.close()

        if connector is not None:
            connector.close()

    def fetch(self, sql, args=None, fetch_type='one', cursor_cls=Cursor):
        connector, cursor = None, None

        try:
            connector, cursor = self.connect(cursor_cls=cursor_cls)
            cursor.execute(sql, args)

        except Exception as e:
            error_mark(e)
            raise e

        else:
            if fetch_type == 'all':
                return cursor.fetchall()

            else:
                return cursor.fetchone()

        finally:
            self.close(connector, cursor)

    def fetch_all(self, sql, args=None, cursor_cls=Cursor):
        return self.fetch(sql, args, 'all', cursor_cls)

    def fetch_one(self, sql, args=None, cursor_cls=Cursor):
        return self.fetch(sql, args, 'one', cursor_cls)

    def dml_trans_handler(self, sql, args=None):
        connector, cursor = None, None

        try:
            connector, cursor = self.connect()
            _return = cursor.execute(sql, args)

        except Exception as e:
            connector.rollback()
            error_mark(e)
            raise e

        else:
            connector.commit()
            return _return

        finally:
            self.close(connector, cursor)

    def __str__(self):
        return "{user}:!password@{host}:{port}/{database}?{charset}".format(
            user=self.mysql_username,
            host=self.mysql_host,
            port=self.mysql_port,
            database=self.mysql_database,
            charset=self.mysql_charset,
        )



# -*- coding: utf-8 -*-
# Author: ZKH
# Date：2021/3/8
from yqn_project_cli.utils.restx import DANamespace
from yqn_project_cli.utils.core.as_flask import JSONResponse

# one module, one namespace, more will be ignored
ns = DANamespace('main', description='', path='/')


"""
import datetime
from yqn_project_cli.utils.core.as_flask import JSONResponse
from yqn_project_cli.utils.decorators import parse_request_with
from api.main.parser import MainIndexParser
from api.main.model import (
    RequestSchema,
    ResponseSchema
)

req = RequestSchema("Request")
res = ResponseSchema("Response")
ns.add_model('Request', req)
ns.add_model('Response', res)


class Swagger:

    @ns.doc(expect=[req])
    @ns.doc(responses={200: ("description", res)})
    @parse_request_with(MainIndexParser())
    def new_add(self, *args, **kwargs):
        return JSONResponse({'now': datetime.datetime.now()})
"""



# -*- coding: utf-8 -*-
# Author: ZKH
# Date：2021/2/24
import os
import json
import types
import errno
import random
import socket
import requests
import termcolor
from requests_toolbelt.multipart.encoder import MultipartEncoder


def merge_iterable_objs(*args):
    _ = []

    for arg in args:
        if isinstance(arg, (list, set, tuple)):
            for _arg in arg:
                _.extend(merge_iterable_objs(_arg))
        else:
            _.append(arg)

    return _


def error_mark(str_):
    print(termcolor.colored(str(str_), 'red'))


def warning_mark(str_):
    print(termcolor.colored(str(str_), 'yellow'))


def health_mark(str_):
    print(termcolor.colored(str(str_), 'green'))


def color_mark(str_, color='blue'):
    print(termcolor.colored(str(str_), str(color)))


def upload_to_oss(target_name, content):
    upload_url = "http://ws.yqn.qa2:30016/api/File/UploadProcess"
    multipart_encoder = MultipartEncoder(
        fields={
            'FileTypeId': '9',
            'Folder': 'file_convert_org',
            'File': ('{file_name}'.format(file_name=target_name), content, 'application/octet-stream')
        },
        boundary='-----------------------------' + str(random.randint(1e28, 1e29 - 1))
    )
    headers = {
        'Content-Type': multipart_encoder.content_type
    }
    try:
        response = requests.post(upload_url, data=multipart_encoder, headers=headers)
    except Exception as e:
        print(e)
        return ''

    response_json = json.loads(response.text)

    if response_json['isSuccess'] and response_json['data'] and response_json['data'][0]['isSuccess']:
        return response_json['data'][0]['fileUrl'][:response_json['data'][0]['fileUrl'].index('?')]  # 文件过期问题临时处理
    else:
        return ''


def load_module_from_pyfile(path, silent=False):
    d = types.ModuleType("route")
    d.__file__ = path
    try:
        with open(path, "rb") as config_file:
            exec(compile(config_file.read(), path, "exec"), d.__dict__)
    except IOError as e:
        if silent and e.errno in (errno.ENOENT, errno.EISDIR, errno.ENOTDIR):
            return False
        e.strerror = "无法读取文件 (%s)" % e.strerror
        raise
    return d


def host_ip():
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(("114.114.114.114", 53))
        ip = s.getsockname()[0]
        s.close()
    except Exception as e:
        error_mark(e)
        return "127.0.0.1"
    else:
        return ip


def host_name():
    return socket.gethostname()


def conf_loader(path):
    config = None
    path = str(path)

    def _config_loader(upper_name, default=None):
        nonlocal config
        upper_name = str(upper_name)
        if config is None and os.path.exists(path):
            config = load_module_from_pyfile(path)

        if getattr(config, upper_name, False):
            return getattr(config, upper_name)

        elif os.environ.get(upper_name, False):
            return os.environ.get(upper_name)

        else:
            return default

    return _config_loader


if __name__ == '__main__':
    print(merge_iterable_objs([1], 4, [5, [6, 7]]))
